﻿using System;
using Ozeki.Media;
using Ozeki.VoIP;

namespace FrequencyToneGeneration
{
    class Program
    {
        static ISoftPhone softphone; // softphone object
        static IPhoneLine phoneLine; // phoneline object
        static IPhoneCall call; // call object

        static PhoneCallAudioSender phoneCallAudioSender;
        static MediaConnector mediaConnector;
        static FrequencyToneGenerator frequencyToneGenerator;

        private static void Main(string[] args)
        {
            // Create a softphone object with RTP port range 5000-10000
            softphone = SoftPhoneFactory.CreateSoftPhone(5000, 10000);

            foreach (var s in softphone.Codecs)
            {
                softphone.DisableCodec(s.PayloadType);
            }
            softphone.EnableCodec(8); //only PCMA codec enabled

            // SIP account registration data, (supplied by your VoIP service provider)
            var registrationRequired = true;
            var userName = "sipusername"; //1000
            var displayName = "sipdisplayname"; //1000
            var authenticationId = "authenticationid"; //1000
            var registerPassword = "Password"; //1000
            var domainHost = "pbxip.voipprovider.com";
            var domainPort = 5060;

            var account = new SIPAccount(registrationRequired, displayName, userName, authenticationId, registerPassword, domainHost, domainPort);

            // Send SIP regitration request
            RegisterAccount(account);

            mediaConnector = new MediaConnector();
            phoneCallAudioSender = new PhoneCallAudioSender();

            frequencyToneGenerator = new FrequencyToneGenerator();
            frequencyToneGenerator.Generate(1400, 5000); // generate an 1400Hz frequency tone, 5 second duration

            // prevents the termination of the application
            Console.ReadLine();
        }

        static void RegisterAccount(SIPAccount account)
        {
            try
            {
                phoneLine = softphone.CreatePhoneLine(account);
                phoneLine.RegistrationStateChanged += sipAccount_RegStateChanged;
                softphone.RegisterPhoneLine(phoneLine);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error during SIP registration: " + ex);
            }
        }

        private static void sipAccount_RegStateChanged(object sender, RegistrationStateChangedArgs e)
        {
            if (e.State == RegState.Error)
                Console.WriteLine("Registration failed!");

            if (e.State == RegState.RegistrationSucceeded)
            {
                Console.WriteLine("Registration succeeded - Online!");
                CreateCall();
            }
        }

        static void CreateCall()
        {
            var numberToDial = "1001";
            call = softphone.CreateCallObject(phoneLine, numberToDial);
            call.CallStateChanged += call_CallStateChanged;
            call.Start();   // SIP INVITE request is being sent
        }

        static void call_CallStateChanged(object sender, CallStateChangedArgs e)
        {
            Console.WriteLine("Call state: {0}.", e);

            if (e.State == CallState.Answered)
                SetupDevices(); // set up the devices by calling the proper method

            if (e.State == CallState.Completed)
                StopDevices(); // detach and disconnect the devices by calling the proper method
        }

        static void SetupDevices()
        {
            phoneCallAudioSender.AttachToCall(call); // attach the phoneCallAudioSender to the call
            mediaConnector.Connect(frequencyToneGenerator, phoneCallAudioSender);
            frequencyToneGenerator.Start(); //start the timer
        }

        static void StopDevices()
        {
            frequencyToneGenerator.Stop();
            mediaConnector.Disconnect(frequencyToneGenerator, phoneCallAudioSender);
            phoneCallAudioSender.Detach();
        }
    }
}
